@@ -10,6 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      UT-Battelle, LLC
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,9 +19,13 @@
  */
 
 #include "ompi_config.h"
+#include <stdio.h>
+
 #include "ompi/datatype/convertor_internal.h"
 #include "ompi/datatype/datatype_internal.h"
 
+#include <stdio.h>
+
 #if OMPI_ENABLE_DEBUG
 extern int ompi_unpack_debug;
 #define DO_DEBUG(INST)  if( ompi_unpack_debug ) { INST }
@@ -186,7 +191,7 @@ ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
                                          size_t* max_data )
 {
     const ompi_datatype_t *pData = pConv->pDesc;
-    char *user_memory, *packed_buffer;
+    unsigned char *user_memory, *packed_buffer;
     uint32_t iov_count, i;
     size_t bConverted, remaining, length, initial_bytes_converted = pConv->bConverted;
     dt_stack_t* stack = pConv->pStack;
@@ -196,7 +201,7 @@ ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
     DO_DEBUG( opal_output( 0, "unpack_homogeneous_contig( pBaseBuf %p, iov_count %d )\n",
                            pConv->pBaseBuf, *out_size ); );
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
-        packed_buffer = (char*)iov[iov_count].iov_base;
+        packed_buffer = (unsigned char*)iov[iov_count].iov_base;
         remaining = pConv->local_size - pConv->bConverted;
         if( remaining > (uint32_t)iov[iov_count].iov_len )
             remaining = iov[iov_count].iov_len;
@@ -284,13 +289,13 @@ ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
  */
 static inline uint32_t
 ompi_unpack_partial_datatype( ompi_convertor_t* pConvertor, dt_elem_desc_t* pElem,
-                              char* partial_data,
+                              unsigned char* partial_data,
                               ptrdiff_t start_position, ptrdiff_t end_position,
-                              char** user_buffer )
+                              unsigned char** user_buffer )
 {
     char unused_byte = 0x7F, saved_data[16];
-    char temporary[16], *temporary_buffer = temporary;
-    char* real_data = *user_buffer + pElem->elem.disp;
+    unsigned char temporary[16], *temporary_buffer = temporary;
+    unsigned char* real_data = *user_buffer + pElem->elem.disp;
     uint32_t i, length, count_desc = 1;
     size_t data_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
 
@@ -356,7 +361,7 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
     dt_elem_desc_t* description;
     dt_elem_desc_t* pElem;
     const ompi_datatype_t *pData = pConvertor->pDesc;
-    char *user_memory_base, *packed_buffer;
+    unsigned char *user_memory_base, *packed_buffer;
     size_t iov_len_local;
     uint32_t iov_count;
 
@@ -385,7 +390,7 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
 
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
 
-        packed_buffer = iov[iov_count].iov_base;
+        packed_buffer = (unsigned char *) iov[iov_count].iov_base;
         iov_len_local = iov[iov_count].iov_len;
         if( 0 != pConvertor->partial_length ) {
             size_t element_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
@@ -421,7 +426,7 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                 type = pElem->elem.common.type;
                 assert( type < DT_MAX_PREDEFINED );
                 if( 0 != iov_len_local ) {
-                    char* temp = user_memory_base;
+                    unsigned char* temp = user_memory_base;
                     /* We have some partial data here. Let's copy it into the convertor
                      * and keep it hot until the next round.
                      */
