@@ -47,7 +47,8 @@ static inline int memchecker_convertor_call (int (*f)(void *, size_t), ompi_conv
         f( (void *)pConvertor->pBaseBuf , pConvertor->local_size);
     } else {
         /* Now we got a noncontigous data. */
-        uint32_t         stack_disp  = 0, elem_pos = 0, i;
+        uint32_t         elem_pos = 0, i;
+        ptrdiff_t        stack_disp  = 0;
         dt_elem_desc_t*  description = pConvertor->use_desc->desc;
         dt_elem_desc_t*  pElem       = &(description[elem_pos]);
         unsigned char   *source_base = pConvertor->pBaseBuf;
@@ -96,10 +97,11 @@ static inline int memchecker_call (int (*f)(void *, size_t), void * addr,
         f( addr , datatype->size * count );
     } else {
         /* Now we got a noncontigous type. */
-        uint32_t         stack_disp  = 0, elem_pos = 0, i;
+        uint32_t         elem_pos = 0, i;
+        ptrdiff_t        stack_disp  = 0;
         dt_elem_desc_t*  description = datatype->opt_desc.desc;
         dt_elem_desc_t*  pElem       = &(description[elem_pos]);
-        unsigned char   *source_base = addr;
+        unsigned char   *source_base = (unsigned char *) addr;
 
         if ( NULL != datatype ) 
             stack_disp = datatype->ub - datatype->lb;
