@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
@@ -16,37 +16,45 @@
  * $HEADER$
  */
 
+#include "ompi_config.h"
 #include "ompi/mca/bml/base/base.h" 
 #include "opal/mca/base/base.h"
 
 #include "opal/mca/mca.h"
 int mca_bml_base_output = -1;
 
-OMPI_DECLSPEC mca_bml_base_module_t mca_bml = {
+mca_bml_base_module_t mca_bml = {
     NULL,                    /* bml_component */
-    0,                       /* bml_eager_limit */ 
-    0,                       /* bml_min_send_size */ 
-    0,                       /* bml_max_send_size */ 
-    0,                       /* bml_min_rdma_size */ 
-    0,                       /* bml_max_rdma_size */ 
     NULL,                    /* bml_add_procs */ 
     NULL,                    /* bml_del_procs */
+    NULL,                    /* bml_add_btl */
+    NULL,                    /* bml_del_btl */
+    NULL,                    /* bml_del_proc_btl */
     NULL,                    /* bml_register */ 
+    NULL,                    /* bml_register_error */ 
     NULL,                    /* bml_finalize*/
-    NULL                    /* bml_progress */
+    NULL                     /* FT event */
 };
 mca_bml_base_component_t mca_bml_component;
 
+static bool init_called = false;
+
+bool
+mca_bml_base_inited(void)
+{
+    return init_called;
+}
 
 int mca_bml_base_init( bool enable_progress_threads, 
-                       bool enable_mpi_threads, 
-                       opal_class_t* endpoint_class) {
+                       bool enable_mpi_threads) {
     opal_list_item_t *item = NULL;
     mca_bml_base_component_t *component = NULL, *best_component = NULL; 
     mca_bml_base_module_t *module = NULL, *best_module = NULL; 
     int priority = 0, best_priority = -1; 
     mca_base_component_list_item_t *cli = NULL; 
 
+    init_called = true;
+
     for (item = opal_list_get_first(&mca_bml_base_components_available); 
          opal_list_get_end(&mca_bml_base_components_available) != item; 
          item = opal_list_get_next(item)) { 
@@ -60,8 +68,7 @@ int mca_bml_base_init( bool enable_progress_threads,
         }
         module = component->bml_init(&priority, 
                                      enable_progress_threads, 
-                                     enable_mpi_threads, 
-                                     endpoint_class); 
+                                     enable_mpi_threads); 
 
         if(NULL == module) { 
             continue; 
@@ -83,7 +90,4 @@ int mca_bml_base_init( bool enable_progress_threads,
                                          &mca_bml_base_components_available, 
                                          (mca_base_component_t*) best_component); 
     }
-
-
 }
-
