@@ -41,49 +41,31 @@ int
 mca_coll_basic_allgather_intra(void *sbuf, int scount,
                                struct ompi_datatype_t *sdtype, void *rbuf,
                                int rcount, struct ompi_datatype_t *rdtype,
-                               struct ompi_communicator_t *comm)
+                               struct ompi_communicator_t *comm,
+                               mca_coll_base_module_t *module)
 {
     int err;
-    char *rbuf_original = NULL, *inplace_temp = NULL;
-    ptrdiff_t true_lb, true_extent, lb, extent;
+    ptrdiff_t lb, extent;
 
     /* Handle MPI_IN_PLACE (see explanantion in reduce.c for how to
        allocate temp buffer) -- note that rank 0 can use IN_PLACE
-       natively, and we'll be ok (actually saves a little bit of
-       copying around) */
-
+       natively, and we can just alias the right position in rbuf
+       as sbuf and avoid using a temporary buffer if gather is
+       implemented correctly */
     if (MPI_IN_PLACE == sbuf && 0 != ompi_comm_rank(comm)) {
-        ompi_ddt_get_extent(rdtype, &lb, &extent);
-        ompi_ddt_get_true_extent(rdtype, &true_lb, &true_extent);
-
-        rbuf_original = (char*)rbuf;
-        sbuf = ((char*) rbuf) + (ompi_comm_rank(comm) * extent * rcount);
-        sdtype = rdtype;
-        scount = rcount;
-
-        inplace_temp = (char*)malloc((true_extent + (rcount - 1) * extent) * 
-                                     ompi_comm_size(comm));
-        if (NULL == inplace_temp) {
-            return OMPI_ERR_OUT_OF_RESOURCE;
-        }
-        rbuf = inplace_temp - lb;
+       ompi_ddt_get_extent(rdtype, &lb, &extent);
+       sbuf = ((char*) rbuf) + (ompi_comm_rank(comm) * extent * rcount);
+       sdtype = rdtype;
+       scount = rcount;
     } 
 
     /* Gather and broadcast. */
 
     err = comm->c_coll.coll_gather(sbuf, scount, sdtype, rbuf, rcount,
-                                   rdtype, 0, comm);
+                                   rdtype, 0, comm, comm->c_coll.coll_gather_module);
     if (MPI_SUCCESS == err) {
         err = comm->c_coll.coll_bcast(rbuf, rcount * ompi_comm_size(comm), 
-                                      rdtype, 0, comm);
-    }
-
-    /* If we've got a temp buffer, copy back out */
-
-    if (MPI_SUCCESS == err && NULL != inplace_temp) {
-        ompi_ddt_copy_content_same_ddt(rdtype, rcount * ompi_comm_size(comm),
-                                       rbuf_original, (char*)rbuf);
-        free(inplace_temp);
+                                      rdtype, 0, comm, comm->c_coll.coll_bcast_module);
     }
 
     /* All done */
@@ -104,18 +86,15 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
                                struct ompi_datatype_t *sdtype,
                                void *rbuf, int rcount,
                                struct ompi_datatype_t *rdtype,
-                               struct ompi_communicator_t *comm)
+                               struct ompi_communicator_t *comm,
+                               mca_coll_base_module_t *module)
 {
-    int rank;
-    int root = 0;
-    int size, rsize;
-    int err;
-    int i;
+    int rank, root = 0, size, rsize, err, i;
     char *tmpbuf = NULL, *ptmp;
-    ptrdiff_t rlb, slb, rextent, sextent;
-    ptrdiff_t incr;
+    ptrdiff_t rlb, slb, rextent, sextent, incr;
     ompi_request_t *req;
-    ompi_request_t **reqs = comm->c_coll_basic_data->mccb_reqs;
+    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
+    ompi_request_t **reqs = basic_module->mccb_reqs;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -213,12 +192,13 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
      */
     if (rank != root) {
         /* post the recv */
-        err = MCA_PML_CALL(recv(rbuf, size * rcount, rdtype, 0,
+        err = MCA_PML_CALL(recv(rbuf, rsize * rcount, rdtype, 0,
                                 MCA_COLL_BASE_TAG_ALLGATHER, comm,
                                 MPI_STATUS_IGNORE));
         if (OMPI_SUCCESS != err) {
             goto exit;
         }
+
     } else {
         /* Send the data to every other process in the remote group
          * except to rank zero. which has it already. */
@@ -230,6 +210,7 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
             if (OMPI_SUCCESS != err) {
                 goto exit;
             }
+
         }
 
         err = ompi_request_wait_all(rsize - 1, reqs, MPI_STATUSES_IGNORE);
