@@ -11,6 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2008      Myricom. All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2009      Cisco systems, Inc.  All rights reserved.
  * 
  * $COPYRIGHT$
  *
@@ -39,6 +40,7 @@
 int mx__regcache_clean(void *ptr, size_t size);
 
 static int ompi_common_mx_initialize_ref_cnt = 0;
+static int ompi_common_mx_available = 0;
 static mca_mpool_base_module_t *ompi_common_mx_fake_mpool = 0;
 
 int
@@ -46,7 +48,7 @@ ompi_common_mx_initialize(void)
 {
     mx_return_t mx_return;
     struct mca_mpool_base_resources_t mpool_resources;
-    int index, value;
+    int index, value, ret = OMPI_SUCCESS;
     
     ompi_common_mx_initialize_ref_cnt++;
     
@@ -65,10 +67,9 @@ ompi_common_mx_initialize(void)
 	   - we have both FREE and MUNMAP support
 	   - we have MUNMAP support and the linux mallopt */
 	value = opal_mem_hooks_support_level();
-	if (((value & (OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT))
-	     == (OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT))
-	    || ((value & OPAL_MEMORY_MUNMAP_SUPPORT) &&
-		OMPI_MPOOL_BASE_HAVE_LINUX_MALLOPT)) {
+        mpool_resources.regcache_clean = mx__regcache_clean;
+	if ((value & (OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT))
+            == (OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT)) {
 	  index = mca_base_param_find("mpi", NULL, "leave_pinned");
 	  if (index >= 0)
             if ((mca_base_param_lookup_int(index, &value) == OPAL_SUCCESS) 
@@ -76,7 +77,6 @@ ompi_common_mx_initialize(void)
 	      
 	      ompi_mpi_leave_pinned = 1;
 	      setenv("MX_RCACHE", "2", 1);
-	      mpool_resources.regcache_clean = mx__regcache_clean;
 	      ompi_common_mx_fake_mpool = 
 		mca_mpool_base_module_create("fake", NULL, &mpool_resources);
 	      if (!ompi_common_mx_fake_mpool) {
@@ -92,16 +92,22 @@ ompi_common_mx_initialize(void)
         mx_return = mx_init(); 
         
         if(MX_SUCCESS != mx_return) {
+            ompi_common_mx_available = -1;
+            if (ompi_common_mx_fake_mpool) {
+                mca_mpool_base_module_destroy(ompi_common_mx_fake_mpool);
+            }
             opal_output(0,
                         "Error in mx_init (error %s)\n",
                         mx_strerror(mx_return));
             return OMPI_ERR_NOT_AVAILABLE;
         }
-        
-    } 
-    return OMPI_SUCCESS;
-}
+        ompi_common_mx_available = 1;
+    } else if (ompi_common_mx_available < 0) {
+        ret = OMPI_ERR_NOT_AVAILABLE;
+    }
 
+    return ret;
+} 
 
 int
 ompi_common_mx_finalize(void)
