@@ -21,23 +21,22 @@
 
 #include "ompi/mca/pml/pml.h"
 #include "ompi/request/request.h"
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+
+BEGIN_C_DECLS
 
 OMPI_DECLSPEC int mca_pml_base_bsend_init(bool enable_mpi_threads);
 OMPI_DECLSPEC int mca_pml_base_bsend_fini(void);
 
-OMPI_DECLSPEC int mca_pml_base_bsend_attach(void* addr, int size);
-OMPI_DECLSPEC int mca_pml_base_bsend_detach(void* addr, int* size);
+int mca_pml_base_bsend_attach(void* addr, int size);
+int mca_pml_base_bsend_detach(void* addr, int* size);
 
 OMPI_DECLSPEC int mca_pml_base_bsend_request_alloc(ompi_request_t*);
 OMPI_DECLSPEC int mca_pml_base_bsend_request_start(ompi_request_t*);
 OMPI_DECLSPEC int mca_pml_base_bsend_request_fini(ompi_request_t*);
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+OMPI_DECLSPEC void*  mca_pml_base_bsend_request_alloc_buf( size_t length );
+OMPI_DECLSPEC int mca_pml_base_bsend_request_free(void* addr);    
 
+END_C_DECLS
 
 #endif
 
