@@ -28,14 +28,25 @@
 #include "ompi/mca/pml/base/pml_base_request.h"
 #include "opal/runtime/opal_progress.h"
 
+
+int mca_pml_base_finalize(void) {
+  if (NULL != mca_pml_base_selected_component.pmlm_finalize) {
+    return mca_pml_base_selected_component.pmlm_finalize();
+  }
+  return OMPI_SUCCESS;
+}
+
+     
 int mca_pml_base_close(void)
 {
     /* turn off the progress code for the pml */
-    opal_progress_unregister(mca_pml.pml_progress);
+    if( NULL != mca_pml.pml_progress ) {
+        opal_progress_unregister(mca_pml.pml_progress);
+    }
 
-     /* Blatently ignore the return code (what would we do to recover,
-        anyway?  This module is going away, so errors don't matter
-        anymore) */
+    /* Blatently ignore the return code (what would we do to recover,
+       anyway?  This module is going away, so errors don't matter
+       anymore) */
 
     /**
      * Destruct the send and receive queues. The ompi_free_list_t destructor
@@ -46,10 +57,7 @@ int mca_pml_base_close(void)
     OBJ_DESTRUCT(&mca_pml_base_recv_requests);
 
     mca_pml.pml_progress = mca_pml_base_progress;
-    if (NULL != mca_pml_base_selected_component.pmlm_finalize) {
-        mca_pml_base_selected_component.pmlm_finalize();
-    }
-
+    
     OBJ_DESTRUCT(&mca_pml_base_pml);
 
     /* Close all remaining available modules (may be one if this is a
