@@ -74,7 +74,7 @@ mca_pml_csum_t mca_pml_csum = {
         mca_pml_csum_start,
         mca_pml_csum_dump,
         mca_pml_csum_ft_event,
-        32768,
+        65535,
         INT_MAX
     }
 };
@@ -194,6 +194,13 @@ int mca_pml_csum_add_comm(ompi_communicator_t* comm)
     if (NULL == pml_comm) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+
+    /* should never happen, but it was, so check */
+    if (comm->c_contextid > (uint32_t) mca_pml_csum.super.pml_max_contextid) {
+        OBJ_RELEASE(pml_comm);
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+    
     mca_pml_csum_comm_init_size(pml_comm, comm->c_remote_group->grp_proc_count);
     comm->c_pml_comm = pml_comm;
 
@@ -293,9 +300,14 @@ int mca_pml_csum_add_procs(ompi_proc_t** procs, size_t nprocs)
     ompi_bitmap_t reachable;
     int rc;
     size_t i;
+    ompi_convertor_t *local_convertor;
 
     if(nprocs == 0)
         return OMPI_SUCCESS;
+    /* Create a convertor for processes on the same node &
+       disable checksum computation for local communication */
+    local_convertor = ompi_convertor_create(ompi_proc_local()->proc_arch, 0);
+    local_convertor->flags &= ~CONVERTOR_WITH_CHECKSUM;
 
     for (i = 0 ; i < nprocs ; ++i) {
         /* we don't have any endpoint data we need to cache on the
@@ -306,8 +318,14 @@ int mca_pml_csum_add_procs(ompi_proc_t** procs, size_t nprocs)
          */
         if (!(OMPI_PROC_FLAG_LOCAL & procs[i]->proc_flags)) {
             procs[i]->proc_convertor->flags |= CONVERTOR_WITH_CHECKSUM;
+        } else {
+            OBJ_RELEASE(procs[i]->proc_convertor);
+            procs[i]->proc_convertor = local_convertor;
+            OBJ_RETAIN(local_convertor);
         }
     }
+    /* Decrement reference count by one, as we increment it twice for ourselves */
+    OBJ_RELEASE(local_convertor);
 
     OBJ_CONSTRUCT(&reachable, ompi_bitmap_t);
     rc = ompi_bitmap_init(&reachable, (int)nprocs);
