@@ -30,22 +30,24 @@
 #endif
 
 #include "opal/types.h"
+#include "ompi/proc/proc.h"
+#include "opal/util/arch.h"
 
-#define MCA_PML_OB1_HDR_TYPE_MATCH     1
-#define MCA_PML_OB1_HDR_TYPE_RNDV      2
-#define MCA_PML_OB1_HDR_TYPE_RGET      3
-#define MCA_PML_OB1_HDR_TYPE_ACK       4
-#define MCA_PML_OB1_HDR_TYPE_NACK      5
-#define MCA_PML_OB1_HDR_TYPE_FRAG      6
-#define MCA_PML_OB1_HDR_TYPE_GET       7
-#define MCA_PML_OB1_HDR_TYPE_PUT       8
-#define MCA_PML_OB1_HDR_TYPE_FIN       9
-#define MCA_PML_OB1_HDR_TYPE_MAX       10
+#define MCA_PML_OB1_HDR_TYPE_MATCH     (MCA_BTL_TAG_PML + 1)
+#define MCA_PML_OB1_HDR_TYPE_RNDV      (MCA_BTL_TAG_PML + 2)
+#define MCA_PML_OB1_HDR_TYPE_RGET      (MCA_BTL_TAG_PML + 3)
+#define MCA_PML_OB1_HDR_TYPE_ACK       (MCA_BTL_TAG_PML + 4)
+#define MCA_PML_OB1_HDR_TYPE_NACK      (MCA_BTL_TAG_PML + 5)
+#define MCA_PML_OB1_HDR_TYPE_FRAG      (MCA_BTL_TAG_PML + 6)
+#define MCA_PML_OB1_HDR_TYPE_GET       (MCA_BTL_TAG_PML + 7)
+#define MCA_PML_OB1_HDR_TYPE_PUT       (MCA_BTL_TAG_PML + 8)
+#define MCA_PML_OB1_HDR_TYPE_FIN       (MCA_BTL_TAG_PML + 9)
 
 #define MCA_PML_OB1_HDR_FLAGS_ACK     1  /* is an ack required */
 #define MCA_PML_OB1_HDR_FLAGS_NBO     2  /* is the hdr in network byte order */
 #define MCA_PML_OB1_HDR_FLAGS_PIN     4  /* is user buffer pinned */
 #define MCA_PML_OB1_HDR_FLAGS_CONTIG  8  /* is user buffer contiguous */
+#define MCA_PML_OB1_HDR_FLAGS_NORDMA  16 /* rest will be send by copy-in-out */
 
 /**
  * Common hdr attributes - must be first element in each hdr type 
@@ -69,7 +71,16 @@ struct mca_pml_ob1_match_hdr_t {
     int32_t  hdr_src;                      /**< source rank */
     int32_t  hdr_tag;                      /**< user tag */
     uint16_t hdr_seq;                      /**< message sequence number */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t  hdr_padding[2];               /**< explicitly pad to 16 bytes.  Compilers seem to already prefer to do this, but make it explicit just in case */
+#endif
 };
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#define OMPI_PML_OB1_MATCH_HDR_LEN  16
+#else
+#define OMPI_PML_OB1_MATCH_HDR_LEN  14
+#endif 
+
 typedef struct mca_pml_ob1_match_hdr_t mca_pml_ob1_match_hdr_t;
 
 #define MCA_PML_OB1_MATCH_HDR_NTOH(h) \
@@ -102,6 +113,9 @@ struct mca_pml_ob1_rendezvous_hdr_t {
 };
 typedef struct mca_pml_ob1_rendezvous_hdr_t mca_pml_ob1_rendezvous_hdr_t;
 
+/* Note that hdr_src_req is not put in network byte order because it
+   is never processed by the receiver, other than being copied into
+   the ack header */
 #define MCA_PML_OB1_RNDV_HDR_NTOH(h) \
     do { \
     MCA_PML_OB1_MATCH_HDR_NTOH((h).hdr_match); \
@@ -119,17 +133,36 @@ typedef struct mca_pml_ob1_rendezvous_hdr_t mca_pml_ob1_rendezvous_hdr_t;
  */
 struct mca_pml_ob1_rget_hdr_t {
     mca_pml_ob1_rendezvous_hdr_t hdr_rndv;
-    ompi_ptr_t hdr_des;                       /**< source descriptor */
     uint32_t hdr_seg_cnt;                     /**< number of segments for rdma */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t hdr_padding[4];
+#endif
+    ompi_ptr_t hdr_des;                       /**< source descriptor */
     mca_btl_base_segment_t hdr_segs[1];       /**< list of segments for rdma */
 };
 typedef struct mca_pml_ob1_rget_hdr_t mca_pml_ob1_rget_hdr_t;
 
+#define MCA_PML_OB1_RGET_HDR_NTOH(h) \
+    do { \
+    MCA_PML_OB1_RNDV_HDR_NTOH((h).hdr_rndv); \
+    (h).hdr_seg_cnt = ntohl((h).hdr_seg_cnt); \
+    } while (0)
+
+#define MCA_PML_OB1_RGET_HDR_HTON(h) \
+    do { \
+    MCA_PML_OB1_RNDV_HDR_HTON((h).hdr_rndv); \
+    (h).hdr_seg_cnt = htonl((h).hdr_seg_cnt); \
+    } while (0) 
+
+
 /**
  *  Header for subsequent fragments.
  */
 struct mca_pml_ob1_frag_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;     /**< common attributes */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t hdr_padding[6];
+#endif
     uint64_t hdr_frag_offset;                /**< offset into message */
     ompi_ptr_t hdr_src_req;                  /**< pointer to source request */
     ompi_ptr_t hdr_dst_req;                  /**< pointer to matched receive */
@@ -155,22 +188,29 @@ typedef struct mca_pml_ob1_frag_hdr_t mca_pml_ob1_frag_hdr_t;
 
 struct mca_pml_ob1_ack_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t hdr_padding[6];
+#endif
     ompi_ptr_t hdr_src_req;                   /**< source request */
     ompi_ptr_t hdr_dst_req;                   /**< matched receive request */
-    uint64_t hdr_rdma_offset;                 /**< starting point rdma protocol */
+    uint64_t hdr_send_offset;                 /**< starting point of copy in/out */
 };
 typedef struct mca_pml_ob1_ack_hdr_t mca_pml_ob1_ack_hdr_t;
 
+/* Note that the request headers are not put in NBO because the
+   src_req is already in receiver's byte order and the dst_req is not
+   used by the receiver for anything other than backpointers in return
+   headers */
 #define MCA_PML_OB1_ACK_HDR_NTOH(h) \
     do { \
-    MCA_PML_OB1_COMMON_HDR_NTOH(h.hdr_common); \
-    (h).hdr_rdma_offset = ntoh64((h).hdr_rdma_offset); \
+    MCA_PML_OB1_COMMON_HDR_NTOH((h).hdr_common); \
+    (h).hdr_send_offset = ntoh64((h).hdr_send_offset); \
     } while (0)
 
 #define MCA_PML_OB1_ACK_HDR_HTON(h) \
     do { \
     MCA_PML_OB1_COMMON_HDR_HTON((h).hdr_common); \
-    (h).hdr_rdma_offset = hton64((h).hdr_rdma_offset); \
+    (h).hdr_send_offset = hton64((h).hdr_send_offset); \
     } while (0) 
 
 /**
@@ -179,27 +219,48 @@ typedef struct mca_pml_ob1_ack_hdr_t mca_pml_ob1_ack_hdr_t;
 
 struct mca_pml_ob1_rdma_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t hdr_padding[2];                   /** two to pad out the hdr to a 4 byte alignment.  hdr_req will then be 8 byte aligned after 4 for hdr_seg_cnt */
+#endif
+    uint32_t hdr_seg_cnt;                     /**< number of segments for rdma */
     ompi_ptr_t hdr_req;                       /**< destination request */
     ompi_ptr_t hdr_des;                       /**< source descriptor */
     uint64_t hdr_rdma_offset;                 /**< current offset into user buffer */ 
-    uint32_t hdr_seg_cnt;                     /**< number of segments for rdma */
     mca_btl_base_segment_t hdr_segs[1];       /**< list of segments for rdma */
 };
 typedef struct mca_pml_ob1_rdma_hdr_t mca_pml_ob1_rdma_hdr_t;
 
+#define MCA_PML_OB1_RDMA_HDR_NTOH(h) \
+    do { \
+    MCA_PML_OB1_COMMON_HDR_NTOH((h).hdr_common); \
+    (h).hdr_seg_cnt = ntohl((h).hdr_seg_cnt); \
+    (h).hdr_rdma_offset = ntoh64((h).hdr_rdma_offset); \
+    } while (0)
+
+#define MCA_PML_OB1_RDMA_HDR_HTON(h) \
+    do { \
+    MCA_PML_OB1_COMMON_HDR_HTON((h).hdr_common); \
+    (h).hdr_seg_cnt = htonl((h).hdr_seg_cnt); \
+    (h).hdr_rdma_offset = hton64((h).hdr_rdma_offset); \
+    } while (0) 
+
 /**
  *  Header used to complete an RDMA operation.
  */
 
 struct mca_pml_ob1_fin_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+    uint8_t hdr_padding[6];
+#endif
     ompi_ptr_t hdr_des;                       /**< completed descriptor */
+    uint32_t hdr_fail;                        /**< RDMA operation failed */
 };
 typedef struct mca_pml_ob1_fin_hdr_t mca_pml_ob1_fin_hdr_t;
 
 #define MCA_PML_OB1_FIN_HDR_NTOH(h) \
     do { \
-    MCA_PML_OB1_COMMON_HDR_NTOH(h.hdr_common); \
+    MCA_PML_OB1_COMMON_HDR_NTOH((h).hdr_common); \
     } while (0)
 
 #define MCA_PML_OB1_FIN_HDR_HTON(h) \
@@ -222,5 +283,88 @@ union mca_pml_ob1_hdr_t {
 };
 typedef union mca_pml_ob1_hdr_t mca_pml_ob1_hdr_t;
 
+#if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+static inline __opal_attribute_always_inline__ void
+ob1_hdr_ntoh(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type)
+{
+    if(!(hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NBO))
+        return;
 
+    switch(hdr_type) {
+        case MCA_PML_OB1_HDR_TYPE_MATCH:
+            MCA_PML_OB1_MATCH_HDR_NTOH(hdr->hdr_match);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_RNDV:
+            MCA_PML_OB1_RNDV_HDR_NTOH(hdr->hdr_rndv);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_RGET:
+            MCA_PML_OB1_RGET_HDR_NTOH(hdr->hdr_rget);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_ACK:
+            MCA_PML_OB1_ACK_HDR_NTOH(hdr->hdr_ack);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_FRAG:
+            MCA_PML_OB1_FRAG_HDR_NTOH(hdr->hdr_frag);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_PUT:
+            MCA_PML_OB1_RDMA_HDR_NTOH(hdr->hdr_rdma);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_FIN:
+            MCA_PML_OB1_FIN_HDR_NTOH(hdr->hdr_fin);
+            break;
+        default:
+            assert(0);
+            break;
+    }
+}
+#else
+#define ob1_hdr_ntoh(h, t) do{}while(0)
+#endif
+
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#define ob1_hdr_hton(h, t, p) \
+    ob1_hdr_hton_intr((mca_pml_ob1_hdr_t*)h, t, p)
+static inline __opal_attribute_always_inline__ void
+ob1_hdr_hton_intr(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type,
+        const ompi_proc_t *proc)
+{
+#ifdef WORDS_BIGENDIAN
+    hdr->hdr_common.hdr_flags |= MCA_PML_OB1_HDR_FLAGS_NBO;
+#else
+
+    if(!(proc->proc_arch & OPAL_ARCH_ISBIGENDIAN))
+        return;
+
+    hdr->hdr_common.hdr_flags |= MCA_PML_OB1_HDR_FLAGS_NBO;
+    switch(hdr_type) {
+        case MCA_PML_OB1_HDR_TYPE_MATCH:
+            MCA_PML_OB1_MATCH_HDR_HTON(hdr->hdr_match);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_RNDV:
+            MCA_PML_OB1_RNDV_HDR_HTON(hdr->hdr_rndv);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_RGET:
+            MCA_PML_OB1_RGET_HDR_HTON(hdr->hdr_rget);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_ACK:
+            MCA_PML_OB1_ACK_HDR_HTON(hdr->hdr_ack);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_FRAG:
+            MCA_PML_OB1_FRAG_HDR_HTON(hdr->hdr_frag);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_PUT:
+            MCA_PML_OB1_RDMA_HDR_HTON(hdr->hdr_rdma);
+            break;
+        case MCA_PML_OB1_HDR_TYPE_FIN:
+            MCA_PML_OB1_FIN_HDR_HTON(hdr->hdr_fin);
+            break;
+        default:
+            assert(0);
+            break;
+    }
+#endif
+}
+#else
+#define ob1_hdr_hton(h, t, p) do{}while(0)
+#endif
 #endif
