@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -35,6 +35,8 @@ static const char FUNC_NAME[] = "MPI_Buffer_attach";
 
 int MPI_Buffer_attach(void *buffer, int size) 
 {
+    int ret = OMPI_SUCCESS;
+
   if (MPI_PARAM_CHECK) {
     OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
     if (NULL == buffer || size < 0) {
@@ -42,6 +44,10 @@ int MPI_Buffer_attach(void *buffer, int size)
     }
   }
 
-  return mca_pml_base_bsend_attach(buffer, size);
+  OPAL_CR_ENTER_LIBRARY();
+  ret = mca_pml_base_bsend_attach(buffer, size);
+
+  OPAL_CR_EXIT_LIBRARY();
+  return ret;
 }
 
