@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -39,7 +40,6 @@ int MPI_Put(void *origin_addr, int origin_count, MPI_Datatype origin_datatype,
             MPI_Datatype target_datatype, MPI_Win win) 
 {
     int rc;
-    if (target_rank == MPI_PROC_NULL) return MPI_SUCCESS;
 
     if (MPI_PARAM_CHECK) {
         rc = OMPI_SUCCESS;
@@ -50,16 +50,29 @@ int MPI_Put(void *origin_addr, int origin_count, MPI_Datatype origin_datatype,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN, FUNC_NAME);
         } else if (origin_count < 0 || target_count < 0) {
             rc = MPI_ERR_COUNT;
-        } else if (ompi_win_peer_invalid(win, target_rank)) {
+        } else if (ompi_win_peer_invalid(win, target_rank) &&
+                   (MPI_PROC_NULL != target_rank)) {
             rc = MPI_ERR_RANK;
         } else if (!ompi_win_comm_allowed(win)) {
-            rc = MPI_ERR_RMA_CONFLICT;
+            rc = MPI_ERR_RMA_SYNC;
+        } else if (NULL == target_datatype || 
+                   MPI_DATATYPE_NULL == target_datatype) {
+            rc = MPI_ERR_TYPE;
+        } else if ( target_disp < 0 ) {
+            rc = MPI_ERR_DISP;
         } else {
-            OMPI_CHECK_DATATYPE_FOR_SEND(rc, origin_datatype, origin_count);
+            OMPI_CHECK_DATATYPE_FOR_ONE_SIDED(rc, origin_datatype, origin_count);
+            if (OMPI_SUCCESS == rc) {
+                OMPI_CHECK_DATATYPE_FOR_ONE_SIDED(rc, target_datatype, target_count);
+            }
         }
         OMPI_ERRHANDLER_CHECK(rc, win, rc, FUNC_NAME);
     }
 
+    if (MPI_PROC_NULL == target_rank) return MPI_SUCCESS;
+
+    OPAL_CR_ENTER_LIBRARY();
+
     rc = win->w_osc_module->osc_put(origin_addr, origin_count, origin_datatype,
                                     target_rank, target_disp, target_count,
                                     target_datatype, win);
