@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +22,7 @@
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/request/request.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Ssend_init = PMPI_Ssend_init
@@ -38,10 +40,12 @@ int MPI_Ssend_init(void *buf, int count, MPI_Datatype type,
                    MPI_Request *request) 
 {
     int rc;
-    if (dest == MPI_PROC_NULL) {
-        *request = &ompi_request_empty;
-        return MPI_SUCCESS;
-    }
+
+    MEMCHECKER(
+        memchecker_datatype(type);
+        memchecker_call(&opal_memchecker_base_isdefined, buf, count, type);
+        memchecker_comm(comm);
+    );
 
     if ( MPI_PARAM_CHECK ) {
         rc = MPI_SUCCESS;
@@ -50,11 +54,12 @@ int MPI_Ssend_init(void *buf, int count, MPI_Datatype type,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, FUNC_NAME);
         } else if (count < 0) {
             rc = MPI_ERR_COUNT;
-        } else if (type == MPI_DATATYPE_NULL) {
+        } else if (MPI_DATATYPE_NULL == type || NULL == type) {
             rc = MPI_ERR_TYPE;
         } else if (tag < 0 || tag > mca_pml.pml_max_tag) {
             rc = MPI_ERR_TAG;
-        } else if (ompi_comm_peer_invalid(comm, dest)) {
+        } else if (ompi_comm_peer_invalid(comm, dest) &&
+                   (MPI_PROC_NULL != dest)) {
             rc = MPI_ERR_RANK;
         } else if (request == NULL) {
             rc = MPI_ERR_REQUEST;
@@ -62,6 +67,24 @@ int MPI_Ssend_init(void *buf, int count, MPI_Datatype type,
         OMPI_ERRHANDLER_CHECK(rc, comm, rc, FUNC_NAME);
     }
 
+    if (MPI_PROC_NULL == dest) {
+        *request = OBJ_NEW(ompi_request_t);
+        /* Other fields were initialized by the constructor for
+           ompi_request_t */
+        (*request)->req_type = OMPI_REQUEST_NOOP;
+        (*request)->req_status = ompi_request_empty.req_status;
+        (*request)->req_complete = true;
+        (*request)->req_state = OMPI_REQUEST_INACTIVE;
+        (*request)->req_persistent = true;
+        (*request)->req_free = ompi_request_persistent_proc_null_free;
+        return MPI_SUCCESS;
+    }
+
+    OPAL_CR_ENTER_LIBRARY();
+
+    /*
+     * Here, we just initialize the request -- memchecker should set the buffer in MPI_Start.
+     */
     rc = MCA_PML_CALL(isend_init(buf,count,type,dest,tag,
                                  MCA_PML_BASE_SEND_SYNCHRONOUS,comm,request));
     OMPI_ERRHANDLER_RETURN(rc, comm, rc, FUNC_NAME);
