@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -37,6 +38,8 @@ MPI_Win MPI_Win_f2c(MPI_Fint win)
 {
     int o_index= OMPI_FINT_2_INT(win);
 
+    OPAL_CR_NOOP_PROGRESS();
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
     }
@@ -46,9 +49,9 @@ MPI_Win MPI_Win_f2c(MPI_Fint win)
        return an invalid C handle. */
 
     if ( 0 > o_index ||
-         o_index >= ompi_pointer_array_get_size(&ompi_mpi_windows)) {
+         o_index >= opal_pointer_array_get_size(&ompi_mpi_windows)) {
         return NULL;
     }
         
-    return (MPI_Win)ompi_pointer_array_get_item(&ompi_mpi_windows, o_index);
+    return (MPI_Win)opal_pointer_array_get_item(&ompi_mpi_windows, o_index);
 }
