@@ -26,8 +26,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
-#include "opal_config.h"
 
+#include "opal_config.h"
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -46,32 +46,40 @@
 #include <unistd.h>
 #endif
 #include <errno.h>
-#include <err.h>
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
-
-#ifdef USE_LOG
-#include "log.h"
-#else
-#define LOG_DBG(x)
-#define log_error	warn
+#ifdef HAVE_UTIL_H
+#include <util.h>
+#endif
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
+#ifdef HAVE_LIBUTIL_H
+#include <libutil.h>
+#endif
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
 #endif
 
-#if defined(HAVE_INTTYPES_H) && !defined(__OpenBSD__)
-#define INTPTR(x)	(intptr_t)x
+/* Some platforms apparently define the udata field of struct kevent as
+ * intptr_t, whereas others define it as void*.  There doesn't seem to be an
+ * easy way to tell them apart via autoconf, so we need to use OS macros. */
+#if defined(HAVE_INTTYPES_H) && !defined(__OpenBSD__) && !defined(__FreeBSD__) && !defined(__darwin__) && !defined(__APPLE__)
+#define PTR_TO_UDATA(x)	((intptr_t)(x))
 #else
-#define INTPTR(x)	x
+#define PTR_TO_UDATA(x)	(x)
 #endif
 
 #include "event.h"
+#include "event-internal.h"
+#include "log.h"
 #include "opal/threads/mutex.h"
 
-extern struct event_list timequeue;
-extern struct event_list eventqueue;
-extern struct event_list addqueue;
 extern opal_mutex_t opal_event_lock;
 
+
+
 #define EVLIST_X_KQINKERNEL	0x1000
 
 #define NEVENT		64
@@ -82,62 +90,84 @@ struct kqop {
 	struct kevent *events;
 	int nevents;
 	int kq;
-} kqueueop;
+};
 
-static void *kq_init	(void);
-static int kq_add	(void *, struct opal_event *);
-static int kq_del	(void *, struct opal_event *);
-static int kq_recalc	(void *, int);
-static int kq_dispatch	(void *, struct timeval *);
+static void *kq_init	(struct event_base *);
+static int kq_add	(void *, struct event *);
+static int kq_del	(void *, struct event *);
+static int kq_dispatch	(struct event_base *, void *, struct timeval *);
 static int kq_insert	(struct kqop *, struct kevent *);
+static void kq_dealloc (struct event_base *, void *);
 
-const struct opal_eventop opal_kqops = {
+const struct eventop kqops = {
 	"kqueue",
 	kq_init,
 	kq_add,
 	kq_del,
-	kq_recalc,
-	kq_dispatch
+	kq_dispatch,
+	kq_dealloc,
+	1 /* need reinit */
 };
 
 static void *
-kq_init(void)
+kq_init(struct event_base *base)
 {
-	int kq;
+    int kq;
+    struct kqop *kqueueop;
 
-	/* Disable kqueue when this environment variable is set */
-	if (getenv("EVENT_NOKQUEUE"))
-		return (NULL);
+    /* Disable kqueue when this environment variable is set */
+    if (getenv("EVENT_NOKQUEUE"))
+        return (NULL);
 
-	memset(&kqueueop, 0, sizeof(kqueueop));
+    if (!(kqueueop = calloc(1, sizeof(struct kqop))))
+        return (NULL);
 
-	/* Initalize the kernel queue */
+    /* Initalize the kernel queue */
 	
-	if ((kq = kqueue()) == -1) {
-		log_error("kqueue");
-		return (NULL);
-	}
+    if ((kq = kqueue()) == -1) {
+        event_warn("kqueue");
+        free (kqueueop);
+        return (NULL);
+    }
 
-	kqueueop.kq = kq;
+    kqueueop->kq = kq;
 
-	/* Initalize fields */
-	kqueueop.changes = malloc(NEVENT * sizeof(struct kevent));
-	if (kqueueop.changes == NULL)
-		return (NULL);
-	kqueueop.events = malloc(NEVENT * sizeof(struct kevent));
-	if (kqueueop.events == NULL) {
-		free (kqueueop.changes);
-		return (NULL);
-	}
-	kqueueop.nevents = NEVENT;
+    /* Initalize fields */
+    kqueueop->changes = malloc(NEVENT * sizeof(struct kevent));
+    if (kqueueop->changes == NULL) {
+        free (kqueueop);
+        return (NULL);
+    }
+    kqueueop->events = malloc(NEVENT * sizeof(struct kevent));
+    if (kqueueop->events == NULL) {
+        free (kqueueop->changes);
+        free (kqueueop);
+        return (NULL);
+    }
+    kqueueop->nevents = NEVENT;
 
-	return (&kqueueop);
-}
+    /* Check for Mac OS X kqueue bug. */
+    kqueueop->changes[0].ident = -1;
+    kqueueop->changes[0].filter = EVFILT_READ;
+    kqueueop->changes[0].flags = EV_ADD;
+    /* 
+     * If kqueue works, then kevent will succeed, and it will
+     * stick an error in events[0].  If kqueue is broken, then
+     * kevent will fail.
+     */
+    if (kevent(kq,
+               kqueueop->changes, 1, kqueueop->events, NEVENT, NULL) != 1 ||
+        (int)kqueueop->events[0].ident != -1 ||
+        kqueueop->events[0].flags != EV_ERROR) {
+        event_warn("%s: detected broken kqueue; not using.", __func__);
+        free(kqueueop->changes);
+        free(kqueueop->events);
+        free(kqueueop);
+        close(kq);
+        return (NULL);
+    }
 
-static int
-kq_recalc(void *arg, int max)
-{
-	return (0);
+    return (kqueueop);
 }
 
 static int
@@ -154,7 +184,7 @@ kq_insert(struct kqop *kqop, struct kevent *kev)
 		newchange = realloc(kqop->changes,
 				    nevents * sizeof(struct kevent));
 		if (newchange == NULL) {
-			log_error("%s: malloc", __func__);
+			event_warn("%s: malloc", __func__);
 			return (-1);
 		}
 		kqop->changes = newchange;
@@ -167,7 +197,7 @@ kq_insert(struct kqop *kqop, struct kevent *kev)
 		 * the next realloc will pick it up.
 		 */
 		if (newresult == NULL) {
-			log_error("%s: malloc", __func__);
+			event_warn("%s: malloc", __func__);
 			return (-1);
 		}
 		kqop->events = newresult;
@@ -177,7 +207,7 @@ kq_insert(struct kqop *kqop, struct kevent *kev)
 
 	memcpy(&kqop->changes[kqop->nchanges++], kev, sizeof(struct kevent));
 
-	LOG_DBG((LOG_MISC, 70, "%s: fd %d %s%s",
+	event_debug(("%s: fd %d %s%s",
 		 __func__, kev->ident, 
 		 kev->filter == EVFILT_READ ? "EVFILT_READ" : "EVFILT_WRITE",
 		 kev->flags == EV_DELETE ? " (del)" : ""));
@@ -192,39 +222,41 @@ kq_sighandler(int sig)
 }
 
 static int
-kq_dispatch(void *arg, struct timeval *tv)
+kq_dispatch(struct event_base *base, void *arg, struct timeval *tv)
 {
 	struct kqop *kqop = arg;
 	struct kevent *changes = kqop->changes;
 	struct kevent *events = kqop->events;
-	struct opal_event *ev;
-	struct timespec ts;
+	struct event *ev;
+	struct timespec ts, *ts_p = NULL;
 	int i, res;
 
-	TIMEVAL_TO_TIMESPEC(tv, &ts);
+	if (tv != NULL) {
+		TIMEVAL_TO_TIMESPEC(tv, &ts);
+		ts_p = &ts;
+	}
+        /* we should release the lock if we're going to enter the
+           kernel in a multi-threaded application.  However, if we're
+           single threaded, there's really no advantage to releasing
+           the lock and it just takes up time we could spend doing
+           something else. */
+        OPAL_THREAD_UNLOCK(&opal_event_lock);
+        
+	res = kevent(kqop->kq, changes, kqop->nchanges,
+	    events, kqop->nevents, ts_p);
+        OPAL_THREAD_LOCK(&opal_event_lock);
 
-        /* release lock while waiting in kernel */
-        if(opal_using_threads()) {
-	    opal_mutex_unlock(&opal_event_lock);
-	    res = kevent(kqop->kq, changes, kqop->nchanges,
-	        events, kqop->nevents, &ts);
-	    opal_mutex_lock(&opal_event_lock);
-        } else {
-	    res = kevent(kqop->kq, changes, kqop->nchanges,
-	        events, kqop->nevents, &ts);
-        }
-
-	kqop->nchanges = 0;
+        kqop->nchanges = 0;
 	if (res == -1) {
 		if (errno != EINTR) {
-			log_error("kevent");
+                        event_warn("kevent");
 			return (-1);
 		}
 
 		return (0);
 	}
 
-	LOG_DBG((LOG_MISC, 80, "%s: kevent reports %d", __func__, res));
+	event_debug(("%s: kevent reports %d", __func__, res));
 
 	for (i = 0; i < res; i++) {
 		int which = 0;
@@ -236,36 +268,40 @@ kq_dispatch(void *arg, struct timeval *tv)
 			 *   closed,
 			 *   ENOENT when the file discriptor was closed and
 			 *   then reopened.
+			 *   EINVAL for some reasons not understood; EINVAL
+			 *   should not be returned ever; but FreeBSD does :-\
 			 * An error is also indicated when a callback deletes
 			 * an event we are still processing.  In that case
 			 * the data field is set to ENOENT.
 			 */
 			if (events[i].data == EBADF ||
+			    events[i].data == EINVAL ||
 			    events[i].data == ENOENT)
 				continue;
+			errno = events[i].data;
+                        event_warn( "kevent failed with error code %d (%s)\n",
+                                    errno, strerror(errno) );
 			return (-1);
 		}
 
-		ev = (struct opal_event *)events[i].udata;
+		ev = (struct event *)events[i].udata;
 
 		if (events[i].filter == EVFILT_READ) {
-			which |= OPAL_EV_READ;
+			which |= EV_READ;
 		} else if (events[i].filter == EVFILT_WRITE) {
-			which |= OPAL_EV_WRITE;
+			which |= EV_WRITE;
 		} else if (events[i].filter == EVFILT_SIGNAL) {
-			which |= OPAL_EV_SIGNAL;
+			which |= EV_SIGNAL;
 		}
 
 		if (!which)
 			continue;
 
-		if (!(ev->ev_events & OPAL_EV_PERSIST)) {
+		if (!(ev->ev_events & EV_PERSIST))
 			ev->ev_flags &= ~EVLIST_X_KQINKERNEL;
-			opal_event_del_i(ev);
-		}
 
-		opal_event_active_i(ev, which,
-		    ev->ev_events & OPAL_EV_SIGNAL ? events[i].data : 1);
+		event_active(ev, which,
+		    ev->ev_events & EV_SIGNAL ? events[i].data : 1);
 	}
 
 	return (0);
@@ -273,40 +309,48 @@ kq_dispatch(void *arg, struct timeval *tv)
 
 
 static int
-kq_add(void *arg, struct opal_event *ev)
+kq_add(void *arg, struct event *ev)
 {
 	struct kqop *kqop = arg;
 	struct kevent kev;
 
-	if (ev->ev_events & OPAL_EV_SIGNAL) {
+	if (ev->ev_events & EV_SIGNAL) {
 		int nsignal = OPAL_EVENT_SIGNAL(ev);
+                struct timespec timeout = { 0, 0 };
 
  		memset(&kev, 0, sizeof(kev));
 		kev.ident = nsignal;
 		kev.filter = EVFILT_SIGNAL;
 		kev.flags = EV_ADD;
-		if (!(ev->ev_events & OPAL_EV_PERSIST))
+		if (!(ev->ev_events & EV_PERSIST))
 			kev.flags |= EV_ONESHOT;
-		kev.udata = (void *) INTPTR(ev);
-		
-		if (kq_insert(kqop, &kev) == -1)
-			return (-1);
+		kev.udata = PTR_TO_UDATA(ev);
 
-		if (signal(nsignal, kq_sighandler) == SIG_ERR)
+		/* Be ready for the signal if it is sent any time between
+		 * now and the next call to kq_dispatch. */
+                if (kevent(kqop->kq, &kev, 1, NULL, 0, &timeout) == -1)
+                	return (-1);
+
+		if (_evsignal_set_handler(ev->ev_base, nsignal,
+					  kq_sighandler) == -1)
 			return (-1);
 
 		ev->ev_flags |= EVLIST_X_KQINKERNEL;
 		return (0);
 	}
 
-	if (ev->ev_events & OPAL_EV_READ) {
+	if (ev->ev_events & EV_READ) {
  		memset(&kev, 0, sizeof(kev));
 		kev.ident = ev->ev_fd;
 		kev.filter = EVFILT_READ;
+#ifdef NOTE_EOF
+		/* Make it behave like select() and poll() */
+		kev.fflags = NOTE_EOF;
+#endif
 		kev.flags = EV_ADD;
-		if (!(ev->ev_events & OPAL_EV_PERSIST))
+		if (!(ev->ev_events & EV_PERSIST))
 			kev.flags |= EV_ONESHOT;
-		kev.udata = (void *) INTPTR(ev);
+		kev.udata = PTR_TO_UDATA(ev);
 		
 		if (kq_insert(kqop, &kev) == -1)
 			return (-1);
@@ -314,14 +358,14 @@ kq_add(void *arg, struct opal_event *ev)
 		ev->ev_flags |= EVLIST_X_KQINKERNEL;
 	}
 
-	if (ev->ev_events & OPAL_EV_WRITE) {
+	if (ev->ev_events & EV_WRITE) {
  		memset(&kev, 0, sizeof(kev));
 		kev.ident = ev->ev_fd;
 		kev.filter = EVFILT_WRITE;
 		kev.flags = EV_ADD;
-		if (!(ev->ev_events & OPAL_EV_PERSIST))
+		if (!(ev->ev_events & EV_PERSIST))
 			kev.flags |= EV_ONESHOT;
-		kev.udata = (void *) INTPTR(ev);
+		kev.udata = PTR_TO_UDATA(ev);
 		
 		if (kq_insert(kqop, &kev) == -1)
 			return (-1);
@@ -333,7 +377,7 @@ kq_add(void *arg, struct opal_event *ev)
 }
 
 static int
-kq_del(void *arg, struct opal_event *ev)
+kq_del(void *arg, struct event *ev)
 {
 	struct kqop *kqop = arg;
 	struct kevent kev;
@@ -341,25 +385,25 @@ kq_del(void *arg, struct opal_event *ev)
 	if (!(ev->ev_flags & EVLIST_X_KQINKERNEL))
 		return (0);
 
-	if (ev->ev_events & OPAL_EV_SIGNAL) {
+	if (ev->ev_events & EV_SIGNAL) {
 		int nsignal = OPAL_EVENT_SIGNAL(ev);
 
  		memset(&kev, 0, sizeof(kev));
-		kev.ident = (int)signal;
+		kev.ident = nsignal;
 		kev.filter = EVFILT_SIGNAL;
 		kev.flags = EV_DELETE;
 		
 		if (kq_insert(kqop, &kev) == -1)
 			return (-1);
 
-		if (signal(nsignal, SIG_DFL) == SIG_ERR)
+		if (_evsignal_restore_handler(ev->ev_base, nsignal) == -1)
 			return (-1);
 
 		ev->ev_flags &= ~EVLIST_X_KQINKERNEL;
 		return (0);
 	}
 
-	if (ev->ev_events & OPAL_EV_READ) {
+	if (ev->ev_events & EV_READ) {
  		memset(&kev, 0, sizeof(kev));
 		kev.ident = ev->ev_fd;
 		kev.filter = EVFILT_READ;
@@ -371,7 +415,7 @@ kq_del(void *arg, struct opal_event *ev)
 		ev->ev_flags &= ~EVLIST_X_KQINKERNEL;
 	}
 
-	if (ev->ev_events & OPAL_EV_WRITE) {
+	if (ev->ev_events & EV_WRITE) {
  		memset(&kev, 0, sizeof(kev));
 		kev.ident = ev->ev_fd;
 		kev.filter = EVFILT_WRITE;
@@ -385,3 +429,18 @@ kq_del(void *arg, struct opal_event *ev)
 
 	return (0);
 }
+
+static void
+kq_dealloc(struct event_base *base, void *arg)
+{
+	struct kqop *kqop = arg;
+
+	if (kqop->changes)
+		free(kqop->changes);
+	if (kqop->events)
+		free(kqop->events);
+	if (kqop->kq)
+		close(kqop->kq);
+	memset(kqop, 0, sizeof(struct kqop));
+	free(kqop);
+}
