@@ -24,7 +24,15 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -33,7 +41,11 @@
 #endif
 
 #if OMPI_WANT_LIBLTDL
-#include "opal/libltdl/ltdl.h"
+  #ifndef __WINDOWS__ 
+    #include "opal/libltdl/ltdl.h" 
+  #else 
+    #include "ltdl.h" 
+  #endif 
 #endif
 
 #include "opal/util/output.h"
@@ -364,6 +376,13 @@ static int save_filename(const char *filename, lt_ptr data)
   strncpy(component_file->filename, filename, OMPI_PATH_MAX);
   component_file->filename[OMPI_PATH_MAX] = '\0';
   component_file->status = UNVISITED;
+
+#if defined(__WINDOWS__) && defined(_DEBUG) 
+  /* remove the debug suffix 'd', otherwise we will fail to  
+     load the module in later phase. */ 
+  component_file->name[strlen(component_file->name)-1] = '\0'; 
+#endif
+
   opal_list_append(&found_files, (opal_list_item_t *) component_file);
 
   /* All done */
@@ -373,6 +392,26 @@ static int save_filename(const char *filename, lt_ptr data)
 }
 
 
+static int file_exists(const char *filename, const char *ext)
+{
+    char *final;
+    struct stat buf;
+    int ret;
+
+    if (NULL != ext) {
+        asprintf(&final, "%s.%s", filename, ext);
+    } else {
+        final = strdup(filename);
+    }
+    if (NULL == final) {
+        return 0;
+    }
+    ret = stat(final, &buf);
+    free(final);
+    return (0 == ret ? 1 : 0);
+}
+
+
 /*
  * Open a component, chasing down its dependencies first, if possible.
  */
@@ -441,13 +480,33 @@ static int open_component(component_file_item_t *target_file,
   component_handle = lt_dlopenext(target_file->filename);
 #endif
   if (NULL == component_handle) {
-    err = strdup(lt_dlerror());
-    opal_output_verbose(vl, 0, "mca: base: component_find: unable to open %s: %s (ignored)", 
-                        target_file->filename, err);
-    free(err);
-    target_file->status = FAILED_TO_LOAD;
-    free_dependency_list(&dependencies);
-    return OPAL_ERR_BAD_PARAM;
+      /* Apparently lt_dlerror() sometimes returns NULL! */
+      const char *str = lt_dlerror();
+      if (NULL != str) {
+          err = strdup(str);
+      } else {
+          err = strdup("lt_dlerror() returned NULL!");
+      }
+      /* Because libltdl erroneously says "file not found" for any
+         type of error -- which is especially misleading when the file
+         is actually there but cannot be opened for some other reason
+         (e.g., missing symbol) -- do some simple huersitics and if
+         the file [probably] does exist, print a slightly better error
+         message. */
+      if (0 == strcmp("file not found", err) &&
+          (file_exists(target_file->filename, "lo") ||
+           file_exists(target_file->filename, "so") ||
+           file_exists(target_file->filename, "dylib") ||
+           file_exists(target_file->filename, "dll"))) {
+          free(err);
+          err = strdup("perhaps a missing symbol, or compiled for a different version of Open MPI?");
+      }
+      opal_output_verbose(vl, 0, "mca: base: component_find: unable to open %s: %s (ignored)", 
+                          target_file->filename, err);
+      free(err);
+      target_file->status = FAILED_TO_LOAD;
+      free_dependency_list(&dependencies);
+      return OPAL_ERR_BAD_PARAM;
   }
 
   /* Successfully opened the component; now find the public struct.
@@ -475,15 +534,20 @@ static int open_component(component_file_item_t *target_file,
 
   component_struct = (mca_base_component_t*)lt_dlsym(component_handle, struct_name);
   if (NULL == component_struct) {
-    opal_output_verbose(vl, 0, "mca: base: component_find: \"%s\" does not appear to be a valid "
-                        "%s MCA dynamic component (ignored)", 
-                        target_file->basename, target_file->type);
-    free(mitem);
-    free(struct_name);
-    lt_dlclose(component_handle);
-    target_file->status = FAILED_TO_LOAD;
-    free_dependency_list(&dependencies);
-    return OPAL_ERR_BAD_PARAM;
+      /* Apparently lt_dlerror() sometimes returns NULL! */
+      const char *str = lt_dlerror();
+      if (NULL == str) {
+          str = "lt_dlerror() returned NULL!";
+      }
+      opal_output_verbose(vl, 0, "mca: base: component_find: \"%s\" does not appear to be a valid "
+			  "%s MCA dynamic component (ignored): %s", 
+			  target_file->basename, target_file->type, str);
+      free(mitem);
+      free(struct_name);
+      lt_dlclose(component_handle);
+      target_file->status = FAILED_TO_LOAD;
+      free_dependency_list(&dependencies);
+      return OPAL_ERR_BAD_PARAM;
   }
 
   /* We found the public struct.  Make sure its MCA major.minor
