@@ -18,6 +18,7 @@
  * $HEADER$
  */
 #include "orte_config.h"
+#include "opal/util/output.h"
 #include "orte/constants.h"
 
 #include <errno.h>
@@ -36,9 +37,7 @@
 #endif
 #endif
 
-#include "orte/util/show_help.h"
 
-#include "orte/mca/oob/base/base.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
@@ -167,7 +166,9 @@ static int hnp_push(const orte_process_name_t* dst_name, orte_iof_tag_t src_tag,
                 np = np / 10;
             }
             /* construct the filename */
-            asprintf(&outfile, "%s.%0*lu", orte_output_filename, numdigs, (unsigned long)proct->name.vpid);
+            asprintf(&outfile, "%s.%d.%0*lu", orte_output_filename,
+                     (int)ORTE_LOCAL_JOBID(proct->name.jobid),
+                     numdigs, (unsigned long)proct->name.vpid);
             /* create the file */
             fdout = open(outfile, O_CREAT|O_RDWR|O_TRUNC, 0644);
             free(outfile);
