@@ -19,12 +19,11 @@
 
 #include "orte_config.h"
 
-#include "orte/util/show_help.h"
-#include "opal/runtime/opal_progress.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_param.h"
 
 #include "orte/mca/rml/rml.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/util/proc_info.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/runtime/orte_globals.h"
