@@ -30,8 +30,8 @@
 
 #include "opal/dss/dss.h"
 
-#include "orte/util/show_help.h"
 #include "orte/mca/rml/rml.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/odls/odls_types.h"
 #include "orte/util/name_fns.h"
