@@ -23,17 +23,19 @@
 #ifndef _MCA_OOB_TCP_MESSAGE_H_
 #define _MCA_OOB_TCP_MESSAGE_H_
 
+#include "orte_config.h"
+#include "orte/types.h"
+
+#include <errno.h>
+
 #include "opal/class/opal_list.h"
+#include "orte/util/show_help.h"
+
 #include "orte/mca/oob/oob.h"
-#include "oob_tcp_peer.h"
 #include "oob_tcp_hdr.h"
-#include <errno.h>
-#include "opal/util/output.h"
-#include "orte/mca/ns/ns_types.h"
+
+BEGIN_C_DECLS
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
 struct mca_oob_tcp_peer_t;
 
 #define MCA_OOB_TCP_IOV_MAX  16
@@ -57,7 +59,7 @@ struct mca_oob_tcp_msg_t {
     int                   msg_rwnum;     /**< number of iovecs left for read/write */
     int                   msg_rwcnt;     /**< total number of iovecs for read/write */
     void*                 msg_rwbuf;     /**< optional buffer for send/recv */
-    mca_oob_callback_fn_t msg_cbfunc;    /**< the callback function for the send/receive */    
+    orte_rml_callback_fn_t msg_cbfunc;    /**< the callback function for the send/receive */    
     void *                msg_cbdata;    /**< the data for the callback fnuction */
     bool                  msg_complete;  /**< whether the message is done sending or not */
     orte_process_name_t   msg_peer;      /**< the name of the peer */
@@ -211,8 +213,7 @@ static inline void mca_oob_tcp_msg_iov_return(mca_oob_tcp_msg_t* msg, struct iov
         free(iov);
 }
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
+
 #endif /* _MCA_OOB_TCP_MESSAGE_H_ */
 
