@@ -86,6 +86,7 @@
 static opal_event_t term_handler;
 static opal_event_t int_handler;
 static opal_event_t pipe_handler;
+static opal_event_t epipe_handler;
 #ifndef __WINDOWS__
 static opal_event_t sigusr1_handler;
 static opal_event_t sigusr2_handler;
@@ -96,6 +97,7 @@ static opal_event_t *orted_exit_event;
 static void shutdown_callback(int fd, short flags, void *arg);
 static void shutdown_signal(int fd, short flags, void *arg);
 static void signal_callback(int fd, short event, void *arg);
+static void epipe_signal_callback(int fd, short flags, void *arg);
 
 static struct {
     bool debug;
@@ -405,6 +407,10 @@ int orte_daemon(int argc, char *argv[])
         return ret;
     }
     
+    /* setup callback for SIGPIPE */
+    opal_signal_set(&epipe_handler, SIGPIPE,
+                    epipe_signal_callback, &epipe_handler);
+    opal_signal_add(&epipe_handler, NULL);
     /* Set signal handlers to catch kill signals so we can properly clean up
      * after ourselves. 
      */
@@ -673,6 +679,9 @@ static void shutdown_callback(int fd, short flags, void *arg)
      */
     orte_odls.kill_local_procs(ORTE_JOBID_WILDCARD, false);
     
+    /* whack any lingering session directory files from our jobs */
+    orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
+    
     /* cleanup the triggers */
     OBJ_DESTRUCT(&orte_exit);
 
@@ -695,11 +704,32 @@ static void shutdown_callback(int fd, short flags, void *arg)
         exit(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
 
+    /* Release all local signal handlers */
+    opal_event_del(&epipe_handler);
+    opal_event_del(&term_handler);
+    opal_event_del(&int_handler);
+#ifndef __WINDOWS__
+    opal_signal_del(&sigusr1_handler);
+    opal_signal_del(&sigusr2_handler);
+#endif  /* __WINDOWS__ */
+
     /* Finalize and clean up ourselves */
     ret = orte_finalize();
     exit(ret);
 }
 
+/**
+ * Deal with sigpipe errors
+ */
+static void epipe_signal_callback(int fd, short flags, void *arg)
+{
+    /* for now, we just announce and ignore them */
+    OPAL_OUTPUT_VERBOSE((1, orte_debug_verbosity,
+                         "%s reports a SIGPIPE error on fd %d",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), fd));
+    return;
+}
+
 static void signal_callback(int fd, short event, void *arg)
 {
     /* just ignore these signals */
